/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef JAC_AP_VISUALIZATION_VISUALIZATION_SKELETON_H
#define JAC_AP_VISUALIZATION_VISUALIZATION_SKELETON_H

#include "ara/com/internal/skeleton/skeleton_adapter.h"
#include "ara/com/internal/skeleton/event_adapter.h"
#include "ara/com/internal/skeleton/field_adapter.h"
#include "ara/com/internal/skeleton/method_adapter.h"
#include "ara/com/crc_verification.h"
#include "jac/ap/visualization/visualization_common.h"
#include <cstdint>

namespace jac {
namespace ap {
namespace visualization {
namespace skeleton {
namespace events
{
    using CameraH265ImagePDataViz = ara::com::internal::skeleton::event::EventAdapter<::CameraH265Image>;
    using LidarHesaiRightPPC2Viz = ara::com::internal::skeleton::event::EventAdapter<::perception::LiDAR2LiDARPerception>;
    using LidarInnoPPC2Viz = ara::com::internal::skeleton::event::EventAdapter<::perception::LiDAR2LiDARPerception>;
    using LidarInno2PPC2Viz = ara::com::internal::skeleton::event::EventAdapter<::perception::LiDAR2LiDARPerception>;
    using LidarHesaiRight2PPC2Viz = ara::com::internal::skeleton::event::EventAdapter<::perception::LiDAR2LiDARPerception>;
    using LidarHesaiLeftPPC2Viz = ara::com::internal::skeleton::event::EventAdapter<::perception::LiDAR2LiDARPerception>;
    using LidarHesaiLeft2PPC2Viz = ara::com::internal::skeleton::event::EventAdapter<::perception::LiDAR2LiDARPerception>;
    static constexpr ara::com::internal::EntityId VisualizationCameraH265ImagePDataVizId = 35045U; //CameraH265ImagePDataViz_event_hash
    static constexpr ara::com::internal::EntityId VisualizationLidarHesaiRightPPC2VizId = 39353U; //LidarHesaiRightPPC2Viz_event_hash
    static constexpr ara::com::internal::EntityId VisualizationLidarInnoPPC2VizId = 24796U; //LidarInnoPPC2Viz_event_hash
    static constexpr ara::com::internal::EntityId VisualizationLidarInno2PPC2VizId = 43181U; //LidarInno2PPC2Viz_event_hash
    static constexpr ara::com::internal::EntityId VisualizationLidarHesaiRight2PPC2VizId = 25020U; //LidarHesaiRight2PPC2Viz_event_hash
    static constexpr ara::com::internal::EntityId VisualizationLidarHesaiLeftPPC2VizId = 36987U; //LidarHesaiLeftPPC2Viz_event_hash
    static constexpr ara::com::internal::EntityId VisualizationLidarHesaiLeft2PPC2VizId = 38317U; //LidarHesaiLeft2PPC2Viz_event_hash
}

namespace methods
{
}

namespace fields
{
}

class VisualizationSkeleton {
private:
    std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> skeletonAdapter;
    void ConstructSkeleton(const ara::com::MethodCallProcessingMode mode)
    {
        if (mode == ara::com::MethodCallProcessingMode::kEvent) {
            if (!(skeletonAdapter->SetMethodThreadNumber(skeletonAdapter->GetMethodThreadNumber(0U), 1024U))) {
#ifndef NOT_SUPPORT_EXCEPTIONS
                ara::core::ErrorCode errorcode(ara::com::ComErrc::kNetworkBindingFailure);
                throw ara::com::ComException(std::move(errorcode));
#else
                std::cerr << "Error: Not support exception, create skeleton failed!" << std::endl;
#endif
            }
        }
        const ara::core::Result<void> resultCameraH265ImagePDataViz = CameraH265ImagePDataViz.Initialize();
        ThrowError(resultCameraH265ImagePDataViz);
        const ara::core::Result<void> resultLidarHesaiRightPPC2Viz = LidarHesaiRightPPC2Viz.Initialize();
        ThrowError(resultLidarHesaiRightPPC2Viz);
        const ara::core::Result<void> resultLidarInnoPPC2Viz = LidarInnoPPC2Viz.Initialize();
        ThrowError(resultLidarInnoPPC2Viz);
        const ara::core::Result<void> resultLidarInno2PPC2Viz = LidarInno2PPC2Viz.Initialize();
        ThrowError(resultLidarInno2PPC2Viz);
        const ara::core::Result<void> resultLidarHesaiRight2PPC2Viz = LidarHesaiRight2PPC2Viz.Initialize();
        ThrowError(resultLidarHesaiRight2PPC2Viz);
        const ara::core::Result<void> resultLidarHesaiLeftPPC2Viz = LidarHesaiLeftPPC2Viz.Initialize();
        ThrowError(resultLidarHesaiLeftPPC2Viz);
        const ara::core::Result<void> resultLidarHesaiLeft2PPC2Viz = LidarHesaiLeft2PPC2Viz.Initialize();
        ThrowError(resultLidarHesaiLeft2PPC2Viz);
    }

    VisualizationSkeleton& operator=(const VisualizationSkeleton&) = delete;

    static void ThrowError(const ara::core::Result<void>& result)
    {
        if (!(result.HasValue())) {
#ifndef NOT_SUPPORT_EXCEPTIONS
            ara::core::ErrorCode errorcode(result.Error());
            throw ara::com::ComException(std::move(errorcode));
#else
            std::cerr << "Error: Not support exception, create skeleton failed!" << std::endl;
#endif
        }
    }
public:
    class ConstructionToken {
    public:
        explicit ConstructionToken(const ara::com::InstanceIdentifier& instanceId,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::visualization::Visualization::ServiceIdentifier, instanceId, mode)),
              CameraH265ImagePDataViz(ptr->GetSkeleton(), events::VisualizationCameraH265ImagePDataVizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceId),
              LidarHesaiRightPPC2Viz(ptr->GetSkeleton(), events::VisualizationLidarHesaiRightPPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceId),
              LidarInnoPPC2Viz(ptr->GetSkeleton(), events::VisualizationLidarInnoPPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceId),
              LidarInno2PPC2Viz(ptr->GetSkeleton(), events::VisualizationLidarInno2PPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceId),
              LidarHesaiRight2PPC2Viz(ptr->GetSkeleton(), events::VisualizationLidarHesaiRight2PPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceId),
              LidarHesaiLeftPPC2Viz(ptr->GetSkeleton(), events::VisualizationLidarHesaiLeftPPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceId),
              LidarHesaiLeft2PPC2Viz(ptr->GetSkeleton(), events::VisualizationLidarHesaiLeft2PPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceId){
        }

        explicit ConstructionToken(const ara::core::InstanceSpecifier& instanceSpec,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::visualization::Visualization::ServiceIdentifier, instanceSpec, mode)),
              CameraH265ImagePDataViz(ptr->GetSkeleton(), events::VisualizationCameraH265ImagePDataVizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceSpec),
              LidarHesaiRightPPC2Viz(ptr->GetSkeleton(), events::VisualizationLidarHesaiRightPPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceSpec),
              LidarInnoPPC2Viz(ptr->GetSkeleton(), events::VisualizationLidarInnoPPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceSpec),
              LidarInno2PPC2Viz(ptr->GetSkeleton(), events::VisualizationLidarInno2PPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceSpec),
              LidarHesaiRight2PPC2Viz(ptr->GetSkeleton(), events::VisualizationLidarHesaiRight2PPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceSpec),
              LidarHesaiLeftPPC2Viz(ptr->GetSkeleton(), events::VisualizationLidarHesaiLeftPPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceSpec),
              LidarHesaiLeft2PPC2Viz(ptr->GetSkeleton(), events::VisualizationLidarHesaiLeft2PPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceSpec){
        }

        explicit ConstructionToken(const ara::com::InstanceIdentifierContainer instanceContainer,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::visualization::Visualization::ServiceIdentifier, instanceContainer, mode)),
              CameraH265ImagePDataViz(ptr->GetSkeleton(), events::VisualizationCameraH265ImagePDataVizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceContainer),
              LidarHesaiRightPPC2Viz(ptr->GetSkeleton(), events::VisualizationLidarHesaiRightPPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceContainer),
              LidarInnoPPC2Viz(ptr->GetSkeleton(), events::VisualizationLidarInnoPPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceContainer),
              LidarInno2PPC2Viz(ptr->GetSkeleton(), events::VisualizationLidarInno2PPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceContainer),
              LidarHesaiRight2PPC2Viz(ptr->GetSkeleton(), events::VisualizationLidarHesaiRight2PPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceContainer),
              LidarHesaiLeftPPC2Viz(ptr->GetSkeleton(), events::VisualizationLidarHesaiLeftPPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceContainer),
              LidarHesaiLeft2PPC2Viz(ptr->GetSkeleton(), events::VisualizationLidarHesaiLeft2PPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceContainer){
        }

        ConstructionToken(ConstructionToken&& other)
            : processMode(std::move(other.processMode)),
              ptr(std::move(other.ptr)),
              CameraH265ImagePDataViz(std::move(other.CameraH265ImagePDataViz)),
              LidarHesaiRightPPC2Viz(std::move(other.LidarHesaiRightPPC2Viz)),
              LidarInnoPPC2Viz(std::move(other.LidarInnoPPC2Viz)),
              LidarInno2PPC2Viz(std::move(other.LidarInno2PPC2Viz)),
              LidarHesaiRight2PPC2Viz(std::move(other.LidarHesaiRight2PPC2Viz)),
              LidarHesaiLeftPPC2Viz(std::move(other.LidarHesaiLeftPPC2Viz)),
              LidarHesaiLeft2PPC2Viz(std::move(other.LidarHesaiLeft2PPC2Viz)){
        }
        ConstructionToken& operator=(ConstructionToken && other)
        {
            if (&other != this) {
                processMode = std::move(other.processMode);
                ptr = std::move(other.ptr);
                CameraH265ImagePDataViz = std::move(other.CameraH265ImagePDataViz);
                LidarHesaiRightPPC2Viz = std::move(other.LidarHesaiRightPPC2Viz);
                LidarInnoPPC2Viz = std::move(other.LidarInnoPPC2Viz);
                LidarInno2PPC2Viz = std::move(other.LidarInno2PPC2Viz);
                LidarHesaiRight2PPC2Viz = std::move(other.LidarHesaiRight2PPC2Viz);
                LidarHesaiLeftPPC2Viz = std::move(other.LidarHesaiLeftPPC2Viz);
                LidarHesaiLeft2PPC2Viz = std::move(other.LidarHesaiLeft2PPC2Viz);
            }
            return *this;
        }
        ConstructionToken(const ConstructionToken&) = delete;
        ConstructionToken& operator = (const ConstructionToken&) = delete;
        std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> GetSkeletonAdapter()
        {
            return std::move(ptr);
        }
        events::CameraH265ImagePDataViz GetCameraH265ImagePDataViz()
        {
            return std::move(CameraH265ImagePDataViz);
        }
        events::LidarHesaiRightPPC2Viz GetLidarHesaiRightPPC2Viz()
        {
            return std::move(LidarHesaiRightPPC2Viz);
        }
        events::LidarInnoPPC2Viz GetLidarInnoPPC2Viz()
        {
            return std::move(LidarInnoPPC2Viz);
        }
        events::LidarInno2PPC2Viz GetLidarInno2PPC2Viz()
        {
            return std::move(LidarInno2PPC2Viz);
        }
        events::LidarHesaiRight2PPC2Viz GetLidarHesaiRight2PPC2Viz()
        {
            return std::move(LidarHesaiRight2PPC2Viz);
        }
        events::LidarHesaiLeftPPC2Viz GetLidarHesaiLeftPPC2Viz()
        {
            return std::move(LidarHesaiLeftPPC2Viz);
        }
        events::LidarHesaiLeft2PPC2Viz GetLidarHesaiLeft2PPC2Viz()
        {
            return std::move(LidarHesaiLeft2PPC2Viz);
        }
        ara::core::Result<void> Initialize()
        {
            ara::core::Result<void> initResult;
            do {
                if (processMode == ara::com::MethodCallProcessingMode::kEvent) {
                    if(!ptr->SetMethodThreadNumber(ptr->GetMethodThreadNumber(0U), 1024U)) {
                        ara::core::ErrorCode errorCode(ara::com::ComErrc::kNetworkBindingFailure);
                        initResult.EmplaceError(errorCode);
                        break;
                    }
                }
                initResult = CameraH265ImagePDataViz.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = LidarHesaiRightPPC2Viz.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = LidarInnoPPC2Viz.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = LidarInno2PPC2Viz.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = LidarHesaiRight2PPC2Viz.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = LidarHesaiLeftPPC2Viz.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = LidarHesaiLeft2PPC2Viz.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
            } while(false);

            return initResult;
        }
    private:
        ara::com::MethodCallProcessingMode processMode;
        std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> ptr;
        events::CameraH265ImagePDataViz CameraH265ImagePDataViz;
        events::LidarHesaiRightPPC2Viz LidarHesaiRightPPC2Viz;
        events::LidarInnoPPC2Viz LidarInnoPPC2Viz;
        events::LidarInno2PPC2Viz LidarInno2PPC2Viz;
        events::LidarHesaiRight2PPC2Viz LidarHesaiRight2PPC2Viz;
        events::LidarHesaiLeftPPC2Viz LidarHesaiLeftPPC2Viz;
        events::LidarHesaiLeft2PPC2Viz LidarHesaiLeft2PPC2Viz;
    };
    explicit VisualizationSkeleton(const ara::com::InstanceIdentifier& instanceId,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        : skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::visualization::Visualization::ServiceIdentifier, instanceId, mode)),
          CameraH265ImagePDataViz(skeletonAdapter->GetSkeleton(), events::VisualizationCameraH265ImagePDataVizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceId),
          LidarHesaiRightPPC2Viz(skeletonAdapter->GetSkeleton(), events::VisualizationLidarHesaiRightPPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceId),
          LidarInnoPPC2Viz(skeletonAdapter->GetSkeleton(), events::VisualizationLidarInnoPPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceId),
          LidarInno2PPC2Viz(skeletonAdapter->GetSkeleton(), events::VisualizationLidarInno2PPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceId),
          LidarHesaiRight2PPC2Viz(skeletonAdapter->GetSkeleton(), events::VisualizationLidarHesaiRight2PPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceId),
          LidarHesaiLeftPPC2Viz(skeletonAdapter->GetSkeleton(), events::VisualizationLidarHesaiLeftPPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceId),
          LidarHesaiLeft2PPC2Viz(skeletonAdapter->GetSkeleton(), events::VisualizationLidarHesaiLeft2PPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceId){
        ConstructSkeleton(mode);
    }

    explicit VisualizationSkeleton(const ara::core::InstanceSpecifier& instanceSpec,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        :skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::visualization::Visualization::ServiceIdentifier, instanceSpec, mode)),
          CameraH265ImagePDataViz(skeletonAdapter->GetSkeleton(), events::VisualizationCameraH265ImagePDataVizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceSpec),
          LidarHesaiRightPPC2Viz(skeletonAdapter->GetSkeleton(), events::VisualizationLidarHesaiRightPPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceSpec),
          LidarInnoPPC2Viz(skeletonAdapter->GetSkeleton(), events::VisualizationLidarInnoPPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceSpec),
          LidarInno2PPC2Viz(skeletonAdapter->GetSkeleton(), events::VisualizationLidarInno2PPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceSpec),
          LidarHesaiRight2PPC2Viz(skeletonAdapter->GetSkeleton(), events::VisualizationLidarHesaiRight2PPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceSpec),
          LidarHesaiLeftPPC2Viz(skeletonAdapter->GetSkeleton(), events::VisualizationLidarHesaiLeftPPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceSpec),
          LidarHesaiLeft2PPC2Viz(skeletonAdapter->GetSkeleton(), events::VisualizationLidarHesaiLeft2PPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceSpec){
        ConstructSkeleton(mode);
    }

    explicit VisualizationSkeleton(const ara::com::InstanceIdentifierContainer instanceContainer,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        :skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::visualization::Visualization::ServiceIdentifier, instanceContainer, mode)),
          CameraH265ImagePDataViz(skeletonAdapter->GetSkeleton(), events::VisualizationCameraH265ImagePDataVizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceContainer),
          LidarHesaiRightPPC2Viz(skeletonAdapter->GetSkeleton(), events::VisualizationLidarHesaiRightPPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceContainer),
          LidarInnoPPC2Viz(skeletonAdapter->GetSkeleton(), events::VisualizationLidarInnoPPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceContainer),
          LidarInno2PPC2Viz(skeletonAdapter->GetSkeleton(), events::VisualizationLidarInno2PPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceContainer),
          LidarHesaiRight2PPC2Viz(skeletonAdapter->GetSkeleton(), events::VisualizationLidarHesaiRight2PPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceContainer),
          LidarHesaiLeftPPC2Viz(skeletonAdapter->GetSkeleton(), events::VisualizationLidarHesaiLeftPPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceContainer),
          LidarHesaiLeft2PPC2Viz(skeletonAdapter->GetSkeleton(), events::VisualizationLidarHesaiLeft2PPC2VizId, ::jac::ap::visualization::Visualization::ServiceIdentifier, instanceContainer){
        ConstructSkeleton(mode);
    }

    VisualizationSkeleton(const VisualizationSkeleton&) = delete;

    VisualizationSkeleton(VisualizationSkeleton&&) = default;
    VisualizationSkeleton& operator=(VisualizationSkeleton&&) = default;
    VisualizationSkeleton(ConstructionToken&& token) noexcept
        : skeletonAdapter(token.GetSkeletonAdapter()),
          CameraH265ImagePDataViz(token.GetCameraH265ImagePDataViz()),
          LidarHesaiRightPPC2Viz(token.GetLidarHesaiRightPPC2Viz()),
          LidarInnoPPC2Viz(token.GetLidarInnoPPC2Viz()),
          LidarInno2PPC2Viz(token.GetLidarInno2PPC2Viz()),
          LidarHesaiRight2PPC2Viz(token.GetLidarHesaiRight2PPC2Viz()),
          LidarHesaiLeftPPC2Viz(token.GetLidarHesaiLeftPPC2Viz()),
          LidarHesaiLeft2PPC2Viz(token.GetLidarHesaiLeft2PPC2Viz()){
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::com::InstanceIdentifier instanceId,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceId, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::core::InstanceSpecifier instanceSpec,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceSpec, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::com::InstanceIdentifierContainer instanceIdContainer,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceIdContainer, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    virtual ~VisualizationSkeleton()
    {
        StopOfferService();
    }

    void OfferService()
    {
        skeletonAdapter->RegisterE2EErrorHandler(&VisualizationSkeleton::E2EErrorHandler, *this);
        skeletonAdapter->OfferService();
    }
    void StopOfferService()
    {
        skeletonAdapter->StopOfferService();
    }
    ara::core::Future<bool> ProcessNextMethodCall()
    {
        return skeletonAdapter->ProcessNextMethodCall();
    }
    bool SetMethodThreadNumber(const std::uint16_t& number, const std::uint16_t& queueSize)
    {
        return skeletonAdapter->SetMethodThreadNumber(number, queueSize);
    }

    virtual void E2EErrorHandler(ara::com::e2e::E2EErrorCode, ara::com::e2e::DataID, ara::com::e2e::MessageCounter){}

    events::CameraH265ImagePDataViz CameraH265ImagePDataViz;
    events::LidarHesaiRightPPC2Viz LidarHesaiRightPPC2Viz;
    events::LidarInnoPPC2Viz LidarInnoPPC2Viz;
    events::LidarInno2PPC2Viz LidarInno2PPC2Viz;
    events::LidarHesaiRight2PPC2Viz LidarHesaiRight2PPC2Viz;
    events::LidarHesaiLeftPPC2Viz LidarHesaiLeftPPC2Viz;
    events::LidarHesaiLeft2PPC2Viz LidarHesaiLeft2PPC2Viz;
};
} // namespace skeleton
} // namespace visualization
} // namespace ap
} // namespace jac

#endif // JAC_AP_VISUALIZATION_VISUALIZATION_SKELETON_H
